/*
------------------------------------------------------------
   Kobo Deluxe - An enhanced SDL port of XKobo
------------------------------------------------------------
 * Copyright (C) 2001, David Olofson
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "form.h"
#include "config.h"
#include "sounds.h"
#include "kobo.h"

kobo_form_t::kobo_form_t()
{
	ypos = 0;
	list = NULL;
	big = 0;
}


kobo_form_t::~kobo_form_t()
{
}


void kobo_form_t::next()
{
	play_sound(SOUND_BEAM);
	ct_form_t::next();
}


void kobo_form_t::prev()
{
	play_sound(SOUND_BEAM);
	ct_form_t::prev();
}


/* virtual */ void kobo_form_t::change(int delta)
{
	ct_form_t::change(delta);
}


/* virtual */void kobo_form_t::build()
{
}


void kobo_form_t::init_widget(ct_widget_t *w)
{
	w->init(engine);
	if(big)
	{
		w->place(x(), y()+ypos, width(), BIG_LINE_H);
		w->font(B_BIG_FONT);
		ypos += BIG_LINE_H;
	}
	else
	{
		w->place(x(), y()+ypos, width(), LINE_H);
		w->font(B_NORMAL_FONT);
		ypos += LINE_H;
	}
	add(w);
}


void kobo_form_t::form_big()
{
	big = 1;
}


void kobo_form_t::form_small()
{
	big = 0;
}


void kobo_form_t::form_begin()
{
	clean();
	ypos = 15;
	list = NULL;
	form_small();
}


void kobo_form_t::form_switch(const char *cap, int *var, int tag)
{
	ct_list_t *w = new ct_list_t(cap);
	w->user = var;
	w->tag = tag;
	w->add("Yes", 1);
	w->add("No", 0);
	init_widget(w);
}


void kobo_form_t::form_spin(const char *cap, int *var, int min, int max, int tag)
{
	ct_spin_t *w = new ct_spin_t(cap, min, max);
	w->user = var;
	w->tag = tag;
	init_widget(w);
}


void kobo_form_t::form_list(const char *cap, int *var, int tag)
{
	ct_list_t *w = new ct_list_t(cap);
	w->user = var;
	w->tag = tag;
	list = w;
	init_widget(w);
}


void kobo_form_t::form_item(const char *cap, int value, int index)
{
	if(list)
	{
		ct_item_t *i = new ct_item_t(cap, value);
		i->index(index);
		list->add(i);
	}
	else
		fprintf(stderr, "kobo_form_t::form_item(): No list!\n");
}


void kobo_form_t::form_button(const char *cap, int tag)
{
	ct_widget_t *w = new ct_button_t(cap);
	w->tag = tag;
	init_widget(w);
}


void kobo_form_t::form_space()
{
	if(big)
		ypos += BIG_SPACE_SIZE;
	else
		ypos += SPACE_SIZE;
}


void kobo_form_t::form_end()
{
}


void kobo_form_t::build_all()
{
	form_begin();
	build();
	form_end();

	/*
	 * Initialize all widgets from prefs.
	 */
	ct_widget_t *w = widgets;
	while(w)
	{
		if(w->user)
		{
			int *val = (int *)w->user;
			w->value(*val);
			DBG(printf("Initialized to %d\n", w->value());)
		}
		w = w->next;
		if(w == widgets)
			break;	//Done!
	}
}


